class << Graphics
  # ウィンドウの横幅（固定幅）
  def width
    640
  end
  # ウィンドウの縦幅（固定幅）
  def height
    480
  end
end

class Bitmap
  # メモリ転送用の関数
  RtlMoveMemory_pi = Win32API.new('kernel32', 'RtlMoveMemory', 'pii', 'i')
  RtlMoveMemory_ip = Win32API.new('kernel32', 'RtlMoveMemory', 'ipi', 'i')
  # ピクセルデータの先頭アドレス
  def data_address
    buffer, ad = "xxxx", object_id * 2 + 16
    RtlMoveMemory_pi.call(buffer, ad, 4); ad = buffer.unpack("L")[0] + 8
    RtlMoveMemory_pi.call(buffer, ad, 4); ad = buffer.unpack("L")[0] + 16
    RtlMoveMemory_pi.call(buffer, ad, 4); return buffer.unpack("L")[0]
  end
end

# Win32API を少し使いやすくするモジュール
module Dll
  def dll_function(func, in_format, out_format = "")
    out_is_bool = ""
    out_modifier = ""
    in_modifier_list = {}
    case out_format
    when "b"
      out_format = "i"
      out_modifier = " != 0"
    end
    in_format.gsub!(/./) do |c|
      i = Regexp.last_match.begin(0)
      case c
      when "f"
        in_modifier_list[i] = "[%s].pack('f').unpack('i')[0]"
        "i"
      when "d"
        in_modifier_list[i] = "[%s].pack('d').unpack('ii')"
        "ii"
      else
        c
      end
    end
    in_modifier = ""
    in_modifier_list.each do |i, imod|
      imod = imod % "argv[#{i}]"
      in_modifier.concat "argv[#{i}] = #{imod}\n"
    end
    module_eval <<-EOS
    FUNC_#{func} = Win32API.new("#{dll_name}", "#{func}", "#{in_format}", "#{out_format}")
    def #{func}(*argv)
      #{in_modifier}
      FUNC_#{func}.call(*argv.flatten) #{out_modifier}
    end
    module_function :#{func}
    EOS
  end
  def dll_name
    self.to_s
  end
end

# 各種関数定義
module User32
  extend Dll
  dll_function "GetForegroundWindow", "", "i"
  dll_function "GetDC", "i", "i"
  dll_function "ReleaseDC", "ii", "i"
  dll_function "InvalidateRect", "iii", "i"
end
module Gdi32
  extend Dll
  dll_function "ChoosePixelFormat", "ip", "i"
  dll_function "SetPixelFormat", "iip", "i"
  dll_function "SwapBuffers", "i", "i"
end
module OpenGL32
  extend Dll
  dll_function "wglCreateContext", "i", "i"
  dll_function "wglMakeCurrent", "ii", "b"
  dll_function "wglDeleteContext", "i"
  dll_function "glClearColor", "ffff"
  dll_function "glClear", "i"
  dll_function "glFlush", ""
  dll_function "glReadPixels", "iiiiiii"
  dll_function "glColor4d", "dddd"
  dll_function "glTexImage2D", "iiiiiiiip"
  dll_function "glTexSubImage2D", "iiiiiiiii"
  dll_function "glEnable", "i"
  dll_function "glDisable", "i"
  dll_function "glTexParameteri", "iii"
  dll_function "glBegin", "i"
  dll_function "glEnd", ""
  dll_function "glTexCoord2f", "ff"
  dll_function "glVertex3i", "iii"
  dll_function "glRotatef", "ffff"
  dll_function "glLoadIdentity", ""
  dll_function "glOrtho", "dddddd"
  dll_function "glTranslatef", "fff"
  dll_function "glMatrixMode", "i"
  dll_function "glScalef", "fff"
  dll_function "glPopMatrix", ""
  dll_function "glPushMatrix", ""
  dll_function "glBindTexture", "ii"
  dll_function "glGenTextures", "ip"
  def glGenTextures(n = 1)
    buf = [0].pack("i") * n
    FUNC_glGenTextures.call(n, buf)
    ret = buf.unpack("i*")
    ret = ret[0] if n == 1
    return ret
  end
  dll_function "glNewList", "ii"
  dll_function "glEndList", ""
  dll_function "glCallList", "i"
  dll_function "glGenLists", "i", "i"
  def glGenLists(n = 1)
    FUNC_glGenLists.call(n)
  end
  dll_function "glDeleteLists", "ii"
  def glDeleteLists(i, range = 1)
    FUNC_glDeleteLists.call(i, range)
  end
  dll_function "glDepthFunc", "i"
  dll_function "glFogi", "ii"
end
module Glu32
  extend Dll
  dll_function "gluPerspective", "dddd"
end

# 各種定数定義
PFD_DRAW_TO_WINDOW = 4
PFD_SUPPORT_OPENGL = 32
PFD_DOUBLEBUFFER = 1
PFD_TYPE_RGBA = 0
PFD_MAIN_PLANE = 0
GL_DEPTH_BUFFER_BIT = 0x00000100
GL_COLOR_BUFFER_BIT = 0x00004000
GL_RGBA = 0x1908
GL_UNSIGNED_BYTE = 0x1401
GL_TEXTURE_ENV = 0x2300
GL_TEXTURE_ENV_MODE = 0x2200
GL_TEXTURE_2D = 0x0DE1
GL_TEXTURE_WRAP_S = 0x2802
GL_TEXTURE_WRAP_T = 0x2803
GL_TEXTURE_MAG_FILTER = 0x2800
GL_TEXTURE_MIN_FILTER = 0x2801
GL_UNPACK_ALIGNMENT = 0x0CF5
GL_LINEAR = 0x2601
GL_CLAMP = 0x2900
GL_MODULATE = 0x2100
GL_QUADS = 7
GL_LINE_LOOP = 2
GL_LINES = 1
GL_PROJECTION = 0x1701
GL_MODELVIEW = 0x1700
GL_COMPILE = 0x1300
GL_LESS = 0x0201
GL_DEPTH_TEST = 0x0B71
GL_FOG = 0x0B60
GL_FOG_MODE = 0x0B65
GL_FOG_START = 0x0B63
GL_FOG_END = 0x0B64

# OpenGL 用の PIXELFORMATDESCRIPTOR
GL_PFD = [
40, 1,
PFD_DRAW_TO_WINDOW | PFD_SUPPORT_OPENGL | PFD_DOUBLEBUFFER,
PFD_TYPE_RGBA, 24,
0, 0, 0, 0, 0, 0,
0, 0, 0,
0, 0, 0, 0,
32, 0, 0,
PFD_MAIN_PLANE, 0,
0, 0, 0
].pack("sslc20l3")

