#==============================================================================
# ■ Window_Mini
#------------------------------------------------------------------------------
# 　アクターの状態を表示するミニウィンドウです。
#==============================================================================
class Window_Mini < Window_Spriteset
  #--------------------------------------------------------------------------
  # ● 定数定義
  #--------------------------------------------------------------------------
  # 必要なビットマップのファイル名
  FILENAMES_BACK = [
  "miniwin/メニュー　非選択時",
  "miniwin/テンション01",
  "miniwin/テンション02",
  "miniwin/テンション03",
  "miniwin/テンション04",
  "miniwin/テンション05"
  ]
  FILENAME_METER_BACK = "miniwin/メーター　バック"
  FILENAME_BASE = "miniwin/メニュー　ベース"
  FILENAME_METER_SP = "miniwin/メーター　特技"
  FILENAME_METER_EP = "miniwin/メーター　術法"
  FILENAME_METER_HP = "miniwin/メーター　HP"
  FILENAME_NAME_BACK = "miniwin/メニュー　名前バック"
  # ウィンドウの大きさ
  WINDOW_WIDTH = 128
  WINDOW_HEIGHT = 128
  # ウィンドウ内の座標, フォントサイズ、位置揃えの方法
  HP_X, HP_Y, HP_W, HP_H, HP_F, HP_A =   78,   96,   40,   20,   20,    2
  LV_X, LV_Y, LV_W, LV_H, LV_F, LV_A =   14,    4,   18,   20,   14,    2
  LP_X, LP_Y, LP_W, LP_H, LP_F, LP_A = LV_X,   19, LV_W, LV_H, LV_F, LV_A
  MB_X, MB_Y, MB_W, MB_H, MB_F, MB_A =    6,   53,   18,   12,   14,    1
  DF_X, DF_Y, DF_W, DF_H, DF_F, DF_A = MB_X,   79, MB_W, MB_H, MB_F, MB_A
  #--------------------------------------------------------------------------
  # ● オブジェクト初期化
  #--------------------------------------------------------------------------
  def initialize(viewport = nil)
    super(viewport, false)
    @ox = -WINDOW_WIDTH / 2
    @oy = -WINDOW_HEIGHT
  end
  #--------------------------------------------------------------------------
  # ● スプライト初期化
  #--------------------------------------------------------------------------
  def setup_sprites
    @sprites = [
    [@back_sprite = Sprite.new(@viewport), 0, 0],
    [@battler_sprite = Sprite_Actor.new(@viewport), 124, 96],
    [@meter_back_sprite = Sprite.new(@viewport), 8, 97],
    [@meter_sp_sprite = Sprite.new(@viewport), 8, 97],
    [@meter_ep_sprite = Sprite.new(@viewport), 8, 106],
    [@meter_hp_sprite = Sprite.new(@viewport), 8, 115],
    [@base_sprite = Sprite.new(@viewport), 0, 0],
    [@name_back_sprite = Sprite.new(@viewport), 0, 0],
    [@draw_sprite = Sprite.new(@viewport), 0, 0]
    ]
  end
  #--------------------------------------------------------------------------
  # ● ビットマップ初期化
  #--------------------------------------------------------------------------
  def setup_bitmaps
    # ビットマップを設定
    set_back
    @meter_back_sprite.bitmap = RPG::Cache.picture(FILENAME_METER_BACK)
    @base_sprite.bitmap = RPG::Cache.picture(FILENAME_BASE)
    @meter_sp_sprite.bitmap = RPG::Cache.picture(FILENAME_METER_SP)
    @meter_ep_sprite.bitmap = RPG::Cache.picture(FILENAME_METER_EP)
    @meter_hp_sprite.bitmap = RPG::Cache.picture(FILENAME_METER_HP)
    @name_back_sprite.bitmap = RPG::Cache.picture(FILENAME_NAME_BACK)
    # 文字描画用ビットマップの作成
    w, h = @base_sprite.bitmap.width, @base_sprite.bitmap.height
    @draw_sprite.bitmap = create_bitmap(w, h)
  end
  #--------------------------------------------------------------------------
  # ● バックのビットマップの設定
  #     tension : テンション
  #--------------------------------------------------------------------------
  def set_back(tension = 0)
    @back_sprite.bitmap = RPG::Cache.picture(FILENAMES_BACK[tension])
  end
  #--------------------------------------------------------------------------
  # ● エフェクト表示中判定
  #--------------------------------------------------------------------------
  def effect?
    @battler_sprite.effect?
  end
  #--------------------------------------------------------------------------
  # ● 解放
  #--------------------------------------------------------------------------
  def dispose
    super
    # !!!テスト用!!!
    @zzz.bitmap.dispose
    @zzz.dispose
    # !!!テスト用ここまで!!!
  end
  #--------------------------------------------------------------------------
  # ● フレーム更新
  #--------------------------------------------------------------------------
  def update
    super
    battler = @battler_sprite.battler
    if battler
      set_pos(battler.screen_x, battler.screen_y)
    end
  end
  #--------------------------------------------------------------------------
  # ● バトラーの設定
  #     visible : バトラー（Game_Actorまたはnil）
  #--------------------------------------------------------------------------
  def battler=(battler)
    return if @battler_sprite.battler == battler
    @battler_sprite.battler = battler
    @battler_status = {}
    if self.visible == !battler
      self.visible = !!battler
    end
    refresh
  end
  #--------------------------------------------------------------------------
  # ● リフレッシュ
  #--------------------------------------------------------------------------
  def refresh
    return #unless @battler_sprite.battler
    [
    [:hp,    HP_X, HP_Y, HP_W, HP_H, HP_F, HP_A],
    [:level, LV_X, LV_Y, LV_W, LV_H, LV_F, LV_A],
    [:lp,    LP_X, LP_Y, LP_W, LP_H, LP_F, LP_A],
    [:mob,   MB_X, MB_Y, MB_W, MB_H, MB_F, MB_A],
    [:dfn,   DF_X, DF_Y, DF_W, DF_H, DF_F, DF_A]
    ].each do |kind, x, y, w, h, f, a|
      if @battler_sprite.battler.send(kind) != @battler_status[kind]
        @battler_status[kind] = @battler_sprite.battler.send(kind)
        @draw_sprite.bitmap.font.size = f
        @draw_sprite.bitmap.fill_rect(x, y, w, h, Color.new(0,0,0,0)) 
        @draw_sprite.bitmap.draw_text(x, y, w, h, @battler_status[kind].to_s, a)
      end
    end
    [
    [@meter_sp_sprite, @battler_sprite.battler.sp, @battler_sprite.battler.maxsp],
    [@meter_ep_sprite, @battler_sprite.battler.ep, @battler_sprite.battler.maxep],
    [@meter_hp_sprite, @battler_sprite.battler.hp, @battler_sprite.battler.maxhp]
    ].each do |sprite, value, max|
      sprite.zoom_x = value.to_f / max
    end
  end
end
